@@ -102,15 +102,30 @@ glade_property_reset_impl (GladeProperty *property)
 gboolean
 glade_property_default_impl (GladeProperty *property)
 {
-	return !g_param_values_cmp (property->class->pspec,
-				    property->value,
-				    property->class->def);
+	return GLADE_PROPERTY_GET_KLASS (property)->equals_value
+		(property, property->class->def);
 }
 
 gboolean
 glade_property_equals_value_impl (GladeProperty *property,
 				  const GValue  *value)
 {
+	if (G_IS_PARAM_SPEC_STRING (property->class->pspec))
+	{
+		const gchar *prop_str, *value_str;
+
+		/* in string specs; NULL and '\0' are 
+		 * treated as equivalent.
+		 */
+		prop_str = g_value_get_string (property->value);
+		value_str = g_value_get_string (value);
+
+		if (prop_str == NULL && value_str && value_str[0] == '\0')
+			return TRUE;
+		else if (value_str == NULL && prop_str && prop_str[0] == '\0')
+			return TRUE;
+	}
+
 	return !g_param_values_cmp (property->class->pspec,
 				    property->value, value);
 }
@@ -120,7 +135,7 @@ glade_property_equals_value_impl (GladeProperty *property,
  * custom set_property method. This includes packing properties.
  */
 static void
-glade_property_set_property (GladeProperty *property, const GValue *value)
+glade_property_sync_property (GladeProperty *property, const GValue *value)
 {
 	if (property->class->packing)
 	{
@@ -137,8 +152,6 @@ glade_property_set_property (GladeProperty *property, const GValue *value)
 	}
 }
 
-extern gboolean glade_widget_dupping;
-
 static void
 glade_property_update_prop_refs (GladeProperty *property, 
 				 const GValue  *old_value,
@@ -201,6 +214,18 @@ glade_property_set_value_impl (GladeProperty *property, const GValue *value)
 	GladeProject *project = property->widget ?
 		glade_widget_get_project (property->widget) : NULL;
 	gboolean      changed = FALSE;
+	GValue old_value = {0,};
+
+#if 0
+	{
+		gchar *str = glade_property_class_make_string_from_gvalue
+			(property->class, value);
+		g_print ("Setting property %s on %s to %s\n",
+			 property->class->id,
+			 property->widget ? property->widget->name : "unknown", str);
+		g_free (str);
+	}
+#endif
 
 	if (!g_value_type_compatible (G_VALUE_TYPE (property->value), G_VALUE_TYPE (value)))
 	{
@@ -209,9 +234,9 @@ glade_property_set_value_impl (GladeProperty *property, const GValue *value)
 		return;
 	}
 
-	if (property->widget &&
+	if (glade_property_superuser () == FALSE &&
+	    property->widget &&
 	    property->class->verify_function &&
-	    glade_widget_is_dupping() == FALSE &&
 	    project && glade_project_is_loading (project) == FALSE)
 	{
 		GObject *object = glade_widget_get_object (property->widget);
@@ -232,6 +257,10 @@ glade_property_set_value_impl (GladeProperty *property, const GValue *value)
 		glade_property_update_prop_refs (property, property->value, value);
 
 
+	/* Make a copy of the old value */
+	g_value_init (&old_value, G_VALUE_TYPE (property->value));
+	g_value_copy (property->value, &old_value);
+	
 	/* Assign property first so that; if the object need be
 	 * rebuilt, it will reflect the new value
 	 */
@@ -242,12 +271,12 @@ glade_property_set_value_impl (GladeProperty *property, const GValue *value)
 
 	if (changed && property->widget)
 	{
-		if (project) glade_project_changed (project);
-
 		g_signal_emit (G_OBJECT (property),
 			       glade_property_signals[VALUE_CHANGED],
-			       0, property->value);
+			       0, &old_value, property->value);
 	}
+	
+	g_value_unset (&old_value);
 }
 
 static void
@@ -299,10 +328,9 @@ glade_property_sync_impl (GladeProperty *property)
 		gboolean  reselect  = FALSE;
 		gboolean  inproject =
 			property->widget->project ?
-			(glade_project_get_widget_by_name
+			(glade_project_has_object
 			 (property->widget->project,
-			  property->widget->name) ? TRUE : FALSE) : FALSE;
-
+			  glade_widget_get_object (property->widget)) ? TRUE : FALSE) : FALSE;
 		
 		if (inproject)
 		{
@@ -334,7 +362,7 @@ glade_property_sync_impl (GladeProperty *property)
 		}
 	}
 	else
-		glade_property_set_property (property, property->value);
+		glade_property_sync_property (property, property->value);
 
 	property->syncing = FALSE;
 }
@@ -347,7 +375,7 @@ glade_property_load_impl (GladeProperty *property)
 	
 	if (property->widget == NULL ||
 	    property->class->packing ||
-	    property->class->atk_type != GPC_ATK_NONE)
+	    property->class->type != GPC_NORMAL)
 		return;
 	object = glade_widget_get_object (property->widget);
 	oclass = G_OBJECT_GET_CLASS (object);
@@ -363,6 +391,7 @@ glade_property_write_impl (GladeProperty  *property,
 {
 	GladePropInfo         info  = { 0, };
 	GladeAtkActionInfo    ainfo = { 0, };
+	GList                *list;
 	gchar                *name, *value, **split, *tmp;
 	gint                  i;
 
@@ -380,7 +409,7 @@ glade_property_write_impl (GladeProperty  *property,
 
 	/* we should change each '-' by '_' on the name of the property 
          * (<property name="...">) */
-	if (property->class->atk_type != GPC_ATK_NONE)
+	if (property->class->type != GPC_NORMAL)
 	{
 
 		tmp = (gchar *)glade_property_class_atk_realname (property->class->id);
@@ -392,31 +421,38 @@ glade_property_write_impl (GladeProperty  *property,
 	}
 
 	/* convert the value of this property to a string */
-	/* XXX Is this right to return here ??? */
-	if ((value = glade_property_class_make_string_from_gvalue 
+	if (property->class->type == GPC_ACCEL_PROPERTY ||
+	    (value = glade_property_class_make_string_from_gvalue 
 	     (property->class, property->value)) == NULL)
+		/* make sure we keep the empty string, also... upcomming
+		 * funcs that may not like NULL.
+		 */
+		value = g_strdup ("");
+	else
 	{
-		g_free (name);
-		return FALSE;
+		/* Escape the string so that it will be parsed as it should. */
+		tmp = value;
+		value = g_markup_escape_text (value, -1);
+		g_free (tmp);
 	}
-
-	switch (property->class->atk_type)
+	
+	switch (property->class->type)
 	{
 	case GPC_ATK_PROPERTY:
 		tmp = g_strdup_printf ("AtkObject::%s", name);
 		g_free (name);
 		name = tmp;
 		/* Dont break here ... */
-	case GPC_ATK_NONE:
-		info.name = alloc_propname(interface, name);
-		info.value = alloc_string(interface,  value);
+	case GPC_NORMAL:
+		info.name = glade_xml_alloc_propname(interface, name);
+		info.value = glade_xml_alloc_string(interface,  value);
 		
 		if (property->class->translatable)
 		{
 			info.translatable = property->i18n_translatable;
 			info.has_context  = property->i18n_has_context;
 			if (property->i18n_comment)
-				info.comment = alloc_string
+				info.comment = glade_xml_alloc_string
 					(interface, property->i18n_comment);
 		}
 		g_array_append_val (props, info);
@@ -426,19 +462,33 @@ glade_property_write_impl (GladeProperty  *property,
 		{
 			for (i = 0; split[i] != NULL; i++)
 			{
-				GladeAtkRelationInfo  rinfo = { 0, };
-				rinfo.type   = alloc_string(interface, name);
-				rinfo.target = alloc_string(interface, split[i]);
+				GladeAtkRelationInfo rinfo = { 0, };
+				rinfo.type   = glade_xml_alloc_string(interface, name);
+				rinfo.target = glade_xml_alloc_string(interface, split[i]);
 				g_array_append_val (props, rinfo);
 			}
 			g_strfreev (split);
 		}
 		break;
 	case GPC_ATK_ACTION:
-		ainfo.action_name = alloc_string(interface, name);
-		ainfo.description = alloc_string(interface, value);
+		ainfo.action_name = glade_xml_alloc_string(interface, name);
+		ainfo.description = glade_xml_alloc_string(interface, value);
 		g_array_append_val (props, ainfo);
 		break;
+	case GPC_ACCEL_PROPERTY:
+		for (list = g_value_get_boxed (property->value); 
+		     list; list = list->next)
+		{
+			GladeAccelInfo *accel = list->data;
+			GladeAccelInfo  accel_info = { 0, };
+
+			accel_info.signal    = glade_xml_alloc_string(interface, accel->signal);
+			accel_info.key       = accel->key;
+			accel_info.modifiers = accel->modifiers;
+
+			g_array_append_val (props, accel_info);
+		}
+		break;
 	default:
 		break;
 	}
@@ -616,7 +666,7 @@ glade_property_klass_init (GladePropertyKlass *prop_class)
 		  _("Whether or not the translatable string has a context prefix"),
 		  FALSE, G_PARAM_READWRITE));
 
-	/* Signals */
+	/* Signal */
 	glade_property_signals[VALUE_CHANGED] =
 		g_signal_new ("value-changed",
 			      G_TYPE_FROM_CLASS (object_class),
@@ -624,8 +674,8 @@ glade_property_klass_init (GladePropertyKlass *prop_class)
 			      G_STRUCT_OFFSET (GladePropertyKlass,
 					       value_changed),
 			      NULL, NULL,
-			      g_cclosure_marshal_VOID__POINTER,
-			      G_TYPE_NONE, 1, G_TYPE_POINTER);
+			      glade_marshal_VOID__POINTER_POINTER,
+			      G_TYPE_NONE, 2, G_TYPE_POINTER, G_TYPE_POINTER);
 
 	glade_property_signals[TOOLTIP_CHANGED] =
 		g_signal_new ("tooltip-changed",
@@ -701,6 +751,7 @@ glade_property_read_packing (GladeProperty      *property,
 					(property, pinfo->has_context);
 				glade_property_i18n_set_comment
 					(property, pinfo->comment);
+
 				property->enabled = TRUE;
 				
 				GLADE_PROPERTY_GET_KLASS (property)->set_value
@@ -816,10 +867,11 @@ glade_property_read_atk_prop (GladeProperty      *property,
 					(property, pinfo->has_context);
 				glade_property_i18n_set_comment
 					(property, pinfo->comment);
-				GLADE_PROPERTY_GET_KLASS (property)->set_value
-					(property, gvalue);
 
 				property->enabled = TRUE;
+
+				GLADE_PROPERTY_GET_KLASS (property)->set_value
+					(property, gvalue);
 			}
 
 			if (free_value)
@@ -901,6 +953,8 @@ glade_property_read_atk_action (GladeProperty      *property,
 
 		if (!strcmp (id, class_id))
 		{
+			/* Need special case for NULL values here ??? */
+			
 			gvalue = glade_property_class_make_gvalue_from_string 
 				(pclass, ainfo->description, project);
 				
@@ -921,6 +975,49 @@ glade_property_read_atk_action (GladeProperty      *property,
 	return gvalue;
 }
 
+static GValue *
+glade_property_read_accel_prop (GladeProperty      *property, 
+				GladePropertyClass *pclass,
+				GladeProject       *project,
+				GladeWidgetInfo    *info,
+				gboolean            free_value)
+{
+	GValue      *gvalue = NULL;
+	GList       *accels = NULL;
+	gint         i;
+
+	for (i = 0; i < info->n_accels; ++i)
+	{
+		GladeAccelInfo *ainfo = info->accels + i;
+		
+		GladeAccelInfo *ainfo_dup = g_new0 (GladeAccelInfo, 1);
+		
+		ainfo_dup            = g_new0 (GladeAccelInfo, 1);
+		ainfo_dup->signal    = g_strdup (ainfo->signal);
+		ainfo_dup->key       = ainfo->key;
+		ainfo_dup->modifiers = ainfo->modifiers;
+
+		accels = g_list_prepend (accels, ainfo_dup);
+	}
+
+	gvalue = g_new0 (GValue, 1);
+	g_value_init (gvalue, GLADE_TYPE_ACCEL_GLIST);
+	g_value_take_boxed (gvalue, accels);
+
+	if (property)
+		GLADE_PROPERTY_GET_KLASS
+			(property)->set_value (property, gvalue);
+	
+	if (free_value)
+	{
+		g_value_unset (gvalue);
+		g_free (gvalue);
+	}
+
+	return gvalue;
+}
+
+
 /*******************************************************************************
                                      API
  *******************************************************************************/
@@ -1027,7 +1124,7 @@ glade_property_default (GladeProperty *property)
  */
 gboolean
 glade_property_equals_value (GladeProperty      *property, 
-			     GValue             *value)
+			     const GValue       *value)
 {
 	g_return_val_if_fail (GLADE_IS_PROPERTY (property), FALSE);
 	return GLADE_PROPERTY_GET_KLASS (property)->equals_value (property, value);
@@ -1259,9 +1356,9 @@ glade_property_read (GladeProperty      *property,
 		ret = glade_property_read_packing 
 			(property, pclass, project, (GladeChildInfo *)info, free_value);
 	}
-	else switch (pclass->atk_type)
+	else switch (pclass->type)
 	{
-	case GPC_ATK_NONE:
+	case GPC_NORMAL:
 		ret = glade_property_read_normal
 			(property, pclass, project, (GladeWidgetInfo *)info, free_value);
 		break;
@@ -1277,6 +1374,10 @@ glade_property_read (GladeProperty      *property,
 		ret = glade_property_read_atk_action
 			(property, pclass, project, (GladeWidgetInfo *)info, free_value);
 		break;
+	case GPC_ACCEL_PROPERTY:
+		ret = glade_property_read_accel_prop
+			(property, pclass, project, (GladeWidgetInfo *)info, free_value);
+		break;
 	default:
 		break;
 	}
@@ -1511,3 +1612,27 @@ glade_property_get_enabled (GladeProperty *property)
 	g_return_val_if_fail (GLADE_IS_PROPERTY (property), FALSE);
 	return property->enabled;
 }
+
+
+static gint glade_property_su_stack = 0;
+
+void
+glade_property_push_superuser (void)
+{
+	glade_property_su_stack++;
+}
+
+void
+glade_property_pop_superuser (void)
+{
+	if (--glade_property_su_stack < 0)
+	{
+		g_critical ("Bug: property super user stack is corrupt.\n");
+	}
+}
+
+gboolean
+glade_property_superuser (void)
+{
+	return glade_property_su_stack > 0;
+}
